//
// Created by fluffymoo on 11.6.18.
//

#ifndef INC_4NEURO_NEURONLOGISTIC_H
#define INC_4NEURO_NEURONLOGISTIC_H

#include <cmath>
#include "Neuron.h"
#include "../constants.h"

class NeuronLogistic:Neuron {
    /**
     * Constructs the object of the Logistic neuron with activation function
     * f(x) = (1 + e^(-x + b))^(-a)
     * @param[in] a First coefficient, stored in activation_function_parameters[0]
     * @param[in] b Second coefficient, stored in activation_function_parameters[1]
     */
    explicit NeuronLogistic(double a = 0.0, double b = 0.0);

    /**
     * Evaluates '(1 + e^(-x + a))^(-b)' and stores the result into the 'state' property
     */
    void activate( ) override;

    /**
     * Calculates the partial derivative of the activation function
     * f(x) = (1 + e^(-x + a))^(-b)
     * @param[in] param_idx Index of the parameter to calculate derivative of
     * @return Partial derivative of the activation function according to the
     * 'param_idx'-th parameter.
     */
    double activation_function_get_partial_derivative(int param_idx) override;

    /**
     * Calculates d/dx of (1 + e^(-x + a))^(-b)
     * @return b * e^(a - x) * [e^(a - x) + 1]^(-b)
     */
    double activation_function_get_derivative( ) override;
};


#endif //INC_4NEURO_NEURONLOGISTIC_H
