//
// Created by fluffymoo on 11.6.18.
//

#ifndef INC_4NEURO_NEURONTANH_H
#define INC_4NEURO_NEURONTANH_H

#include <cmath>
#include "Neuron.h"
#include "../constants.h"

class NeuronTanh:Neuron {
    /**
     * Constructs the object of the Hyperbolic Tangent >neuron with activation function
     * f(x) = (e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))
     * @param[in] a First coefficient, stored in activation_function_parameters[0]
     */
    explicit NeuronTanh(double a = 0.0);

    /**
     * Evaluates '(e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))' and stores the result into the 'state' property
     */
    void activate( ) override;

    /**
     * Calculates the partial derivative of the activation function
     * f(x) = (e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))
     * @param[in] param_idx Index of the parameter to calculate derivative of
     * @return Partial derivative of the activation function according to the
     * 'param_idx'-th parameter.
     */
    double activation_function_get_partial_derivative(int param_idx) override;

    /**
     * Calculates d/dx of (e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))
     * @return a * e^(b - x) * [e^(b - x) + 1]^(-a)
     */
    double activation_function_get_derivative( ) override;
};


#endif //INC_4NEURO_NEURONTANH_H
